from aipyapp.display import DisplayPlugin

class DisplayNull(DisplayPlugin):
    """Null display style"""
    name = "null"
    version = "1.0.0"
    description = "Null display style"
    author = "AiPy Team"
    pass
